/* r4istrap.c
 * (c) 2010 Trini
 * Extension of fincs' work, generates a clean R4i-SDHC binary for HBMenu
 * Free-as-in-beer license, do what you want, give credits if you feel like it.
 */

#include <stdio.h>
#include <math.h>

int R4size;
unsigned int R4arm7size;
unsigned int R4arm9size;
unsigned int R4arm7offset;
unsigned int R4arm9offset;
unsigned int R4fatoffset;

int fsize (FILE* file) {
	fseek (file, 0, SEEK_END);
	int filesize = ftell(file);
	rewind(file);
	return filesize;
}

void parse_header(FILE* file) {
	// Get size of dirty R4.dat
	R4size = fsize (file);
	printf("Old R4size: %i\n",R4size);

	// Get the ARM9 code offset
	fseek (file, 32, SEEK_SET);
	fread(&R4arm9offset, 1, 4, file);
	printf("ARM9 ROM Offset: %i\n", R4arm9offset);

	// Get the ARM9 code size
	fseek (file, 44, SEEK_SET);
	fread(&R4arm9size, 1, 4, file);
	printf("ARM9 ROM Size: %i\n", R4arm9size);

	// Get the ARM7 code offset
	fseek (file, 48, SEEK_SET);
	fread(&R4arm7offset, 1, 4, file);
	printf("ARM7 ROM Offset: %i\n", R4arm7offset);
}


int main (int argc, char *argv[]) {

	FILE* old_dat = fopen("R4.dat", "rb");
	FILE* new_dat = fopen("R4i.dat", "w+b");
	FILE* arm7 = fopen("arm7.bin", "rb");
	FILE* arm9 = fopen("arm9.bin", "rb");
	FILE* payload = fopen("payload.bin", "rb");
	
	printf("Files opened...\n");
	parse_header(old_dat);

	// write the new zero'd file
	int i;
	int R4sectors = ceil((double)R4arm9size/ 512);
	printf ("Arm9 sectors: %i\n", R4sectors);
	int newSize = 512+(512*R4sectors)+512+512;
	printf ("Newsize: %i\n", newSize);
	char foo = 0x0;
	for (i = 0; i < newSize; i++) {
		fwrite(&foo,1,1,new_dat);
	}

	// validate and drop in arm7 spin code
	int a7size = fsize(arm7);
	if ( a7size-8 <= 504) {
		char a7buf[a7size-8];
		fseek(arm7,8,SEEK_SET);
		fread(a7buf,1,a7size-8,arm7);
		fseek(new_dat,R4arm7offset+8,SEEK_SET);
		fwrite(a7buf,1,a7size-8,new_dat);
	}
	else {
		printf ("ARM7 spin binary larger than 512 bytes.\n This is not accounted for, roll your own injector.\n");
		exit (-1);
	}

	// validate and drop in arm9 loader
	int a9size = fsize(arm9);
	if (a9size-8 <= 236) {
		char a9buf[a9size-8];
		fseek(arm9,8,SEEK_SET);
		fread(a9buf,1,a9size-8,arm9);
		fseek(new_dat,R4arm9offset+8, SEEK_SET);
		fwrite(a9buf,1,a9size-8,new_dat);

		// write magic bytes << MUST GET RID OF THIS SOMEHOW...
		fseek(new_dat, R4arm9offset+236, SEEK_SET);
		char magicbytes[] = {0x49, 0x00, 0x00, 0xEB};
		fwrite (magicbytes, 1, 4, new_dat);
	}
	else {
		printf ("ARM9 payload loader larger than 228 bytes.\n This is not accounted for, roll your own injector.\n");
		exit (-1);
	}

	// validate and drop in payload
	int payloadsize = fsize(payload);
	
	if (payloadsize <= R4sectors*506) {
		char payloadbuf[506];
		fseek(new_dat, R4arm9offset+512, SEEK_SET);
		fseek(payload, 0, SEEK_SET);

		for (i = 0; i < payloadsize; i+=506) {
			fread(payloadbuf, 1, 506, payload);
			fseek(new_dat, 6, SEEK_CUR);
			fwrite(payloadbuf, 1, 506, new_dat);
		}
	}
	else {
		printf ("ARM9 payload larger than original R4.\n This is not accounted for, roll your own injector.\n");
		exit (-1);
	}

	// insert dummy FNT
	fseek(new_dat, R4arm7offset+512, SEEK_SET);
	char fntbuf[] = {0x08,0x0,0x0,0x0,0x0,0x0,0x1};
	fwrite (fntbuf,1,7,new_dat);

	// generate header
	char headerbuf[512];
	fseek (new_dat, 0, SEEK_SET);
	fseek (old_dat, 0, SEEK_SET);
	fread (headerbuf, 1, 512, old_dat);
	fwrite (headerbuf, 1, 512, new_dat);

	// fix a7 code size
	fseek (new_dat, 60, SEEK_SET);
	fwrite(&a7size, 1, 4, new_dat);

	// fix fnt and fat offsets
	fseek (new_dat, 64, SEEK_SET);
	int fntoff = R4arm7offset+512;
	int fatoff = fntoff+512;
	fwrite (&fntoff, 1, 4, new_dat);
	fseek (new_dat, 4, SEEK_CUR);
	fwrite (&fatoff, 1, 4, new_dat);
	fseek (new_dat, 128, SEEK_SET);
	fwrite (&fatoff, 1, 4, new_dat);

	fclose( old_dat);
	fclose (new_dat);
	fclose (arm9);
	fclose (arm7);
	fclose (payload);

	return 0;
}
